// src/dsiUnitNode.js
import { dsiPrefixesHTML, dsiUnitsHTML } from "./unitStrings.js";

export class DSIUnitNode {
  constructor(prefix, unit, exponent = '', valid = true, scaleFactor = 1.0) {
    this.prefix = prefix;
    this.unit = unit;
    this.valid = valid;
    if (exponent === '' || exponent === '1') {
      this.exponent = 1;
    } else {
      const numExp = Number(exponent);
      this.exponent = isNaN(numExp) ? exponent : numExp;
    }
    this.scaleFactor = scaleFactor;
  }

  toHTML(options = {}) {
    let prefixHTML = '';
    if (this.prefix) {
      if (this.prefix in dsiPrefixesHTML) {
        prefixHTML = `<span class="dsi-unit">${dsiPrefixesHTML[this.prefix]}</span>`;
      } else {
        prefixHTML = `<span class="dsi-unit"><span class="dsi-invalid">${this.prefix}</span></span>`;
      }
    }
    let unitHTML = '';
    if (this.unit) {
      if (this.unit in dsiUnitsHTML) {
        unitHTML = `<span class="dsi-unit">${dsiUnitsHTML[this.unit]}</span>`;
      } else {
        unitHTML = `<span class="dsi-unit"><span class="dsi-invalid">${this.unit}</span></span>`;
      }
    }
    let baseHTML = prefixHTML + unitHTML;
    if (this.exponent !== 1) {
      if (typeof this.exponent === 'number') {
        if (Math.abs(this.exponent - 0.5) < 0.0001) {
          // Square root.
          return options.oneLine ? `√(${baseHTML})`
              : `<span class="dsi-root"><span class="dsi-root-symbol">√</span><span class="dsi-root-content">${baseHTML}</span></span>`;
        } else if (Math.abs(this.exponent - 0.333333) < 0.0001) {
          // Cube root.
          return options.oneLine ? `∛(${baseHTML})`
              : `<span class="dsi-root"><span class="dsi-root-symbol">∛</span><span class="dsi-root-content">${baseHTML}</span></span>`;
        } else if (Math.abs(this.exponent - 0.25) < 0.0001) {
          // Fourth root.
          return options.oneLine ? `∜(${baseHTML})`
              : `<span class="dsi-root"><span class="dsi-root-symbol">∜</span><span class="dsi-root-content">${baseHTML}</span></span>`;
        } else {
          return options.oneLine
              ? `${baseHTML}^(${this.exponent})`
              : `${baseHTML}<sup class="dsi-exponent">${this.exponent}</sup>`;
        }
      } else {
        return options.oneLine
            ? `${baseHTML}^(<span class="dsi-invalid">${this.exponent}</span>)`
            : `${baseHTML}<sup class="dsi-exponent"><span class="dsi-invalid">${this.exponent}</span></sup>`;
      }
    }
    return baseHTML;
  }

  toString() {
    let str = '';
    if (this.prefix) str += '\\' + this.prefix;
    str += '\\' + this.unit;
    if (this.exponent !== 1) str += `\\tothe{${this.exponent}}`;
    return str;
  }
}
