/**
 * @jest-environment jsdom
 */
import "../src/dsiUnitInput.js";
import { DSIUnit } from "../src/dsiUnit.js";

describe("DSIUnitInput custom element", () => {
    let element;
    beforeEach(() => {
        document.body.innerHTML = `<dsi-unit-input></dsi-unit-input>`;
        element = document.querySelector("dsi-unit-input");
    });

    test("initializes with an input field", () => {
        const input = element.shadowRoot.querySelector("#dsiInput");
        expect(input).not.toBeNull();
    });

    test("shows suggestions after typing backslash and letters", () => {
        const input = element.shadowRoot.querySelector("#dsiInput");
        input.value = "\\m";
        input.dispatchEvent(new Event("input"));
        const suggestions = element.shadowRoot.querySelectorAll(".autocomplete-item");
        expect(suggestions.length).toBeGreaterThan(0);
        suggestions.forEach(item => {
            expect(item.textContent).toMatch(/^\\m/);
        });
    });

    test("on blur, input is replaced by rendered output", () => {
        const input = element.shadowRoot.querySelector("#dsiInput");
        const display = element.shadowRoot.querySelector("#dsiDisplay");
        input.value = "\\metre";
        input.dispatchEvent(new Event("input"));
        input.dispatchEvent(new Event("blur"));
        expect(input.style.display).toBe("none");
        expect(display.style.display).toBe("block");
        const unit = new DSIUnit("\\metre");
        expect(display.innerHTML).toBe(unit.toHTML());
    });

    test("on focus, raw value is restored", () => {
        const input = element.shadowRoot.querySelector("#dsiInput");
        input.value = "\\metre\\tothe{2}";
        input.dispatchEvent(new Event("input"));
        input.dispatchEvent(new Event("blur"));
        input.dispatchEvent(new Event("focus"));
        expect(input.style.display).toBe("block");
        expect(input.value).toBe("\\metre\\tothe{2}");
    });

    test("external suggestions list overrides defaults", () => {
        // Set the attribute "suggestions-list" to only allow "foo" and "bar"
        element.setAttribute("suggestions-list", "foo,bar");
        // Force re-read of the attribute by calling attributeChangedCallback manually if needed.
        const input = element.shadowRoot.querySelector("#dsiInput");
        input.value = "\\f";
        input.dispatchEvent(new Event("input"));
        const suggestions = element.shadowRoot.querySelectorAll(".autocomplete-item");
        expect(suggestions.length).toBeGreaterThan(0);
        suggestions.forEach(item => {
            expect(item.textContent).toMatch(/^\\(foo|bar)$/);
        });
    });
});
