// tests/dsiUnits.test.js
import { DSIUnit } from '../src/dsiUnit.js';

describe('DSIUnit HTML Output Tests', () => {
  test('Base case: one unit without prefix or exponent', () => {
    const unit = new DSIUnit('\\metre');
    expect(unit.tree.length).toBe(1);
    expect(unit.tree[0].length).toBe(1);
    expect(unit.tree[0][0].unit).toBe('metre');
    expect(unit.tree[0][0].exponent).toBe(1);
    expect(unit.toHTML()).toBe('<span class="dsi-unit-wrapper"><span class="dsi-unit">m</span></span>');
    expect(unit.valid).toBe(true);
    expect(unit.warnings).toEqual([]);
  });

  test('Exponent handling: metre^2 and square-root', () => {
    const unit2 = new DSIUnit('\\metre\\tothe{2}');
    expect(unit2.toHTML()).toBe('<span class="dsi-unit-wrapper"><span class="dsi-unit">m</span><sup class="dsi-exponent">2</sup></span>');

    const unitSqrt = new DSIUnit('\\metre\\tothe{0.5}');
    expect(unitSqrt.toHTML()).toBe('<span class="dsi-unit-wrapper"><span class="dsi-root"><span class="dsi-root-symbol">√</span><span class="dsi-root-content"><span class="dsi-unit">m</span></span></span></span>');
  });

  test('Fraction: e.g. mega metre per second^2', () => {
    const unitFrac = new DSIUnit('\\mega\\metre\\per\\second\\tothe{2}');
    const expectedHTML = '<span class="dsi-unit-wrapper"><span class="dsi-fraction"><span class="dsi-numerator"><span class="dsi-unit">M</span><span class="dsi-unit">m</span></span><span class="dsi-denom"><span class="dsi-unit">s</span><sup class="dsi-exponent">2</sup></span></span></span>';
    expect(unitFrac.toHTML()).toBe(expectedHTML);
  });

  test('Robustness: unknown unit', () => {
    const unitUnknown = new DSIUnit('\\foo');
    const expectedHTML = '<span class="dsi-unit-wrapper" title="The identifier «foo» does not match any D-SI units!"><span class="dsi-error"><span class="dsi-unit"><span class="dsi-invalid">foo</span></span></span></span>';
    expect(unitUnknown.toHTML()).toBe(expectedHTML);
    expect(unitUnknown.valid).toBe(false);
    expect(unitUnknown.warnings).toContain('The identifier «foo» does not match any D-SI units!');
  });

  test('Non D-SI unit marker', () => {
    const nonDsi = new DSIUnit('|NonDsiUnit');
    const expectedHTML = '<span class="dsi-unit-wrapper"><span class="dsi-nonunit">NonDsiUnit</span></span>';
    expect(nonDsi.toHTML()).toBe(expectedHTML);
  });

  test('Empty string', () => {
    const emptyUnit = new DSIUnit('');
    const expectedHTML = '<span class="dsi-unit-wrapper"><span class="dsi-nonunit">NULL</span></span>';
    expect(emptyUnit.toHTML()).toBe(expectedHTML);
    expect(emptyUnit.warnings).toContain('Given D-SI string is empty!');
  });

  test('Double backslash handling', () => {
    const unitDouble = new DSIUnit('\\\\metre\\per\\second');
    const expectedHTML = '<span class="dsi-unit-wrapper" title="Double backslash found in string, treating as one backslash: «\\metre\\per\\second»"><span class="dsi-error"><span class="dsi-fraction"><span class="dsi-numerator"><span class="dsi-unit">m</span></span><span class="dsi-denom"><span class="dsi-unit">s</span></span></span></span></span>';
    expect(unitDouble.toHTML()).toBe(expectedHTML);
    expect(unitDouble.warnings.some(w => w.includes('Double backslash'))).toBe(true);
  });

  test('Fourth root: micro metre tothe{0.25}', () => {
    const unitFourth = new DSIUnit('\\micro\\metre\\tothe{0.25}');
    const expectedHTML = '<span class="dsi-unit-wrapper"><span class="dsi-root"><span class="dsi-root-symbol">∜</span><span class="dsi-root-content"><span class="dsi-unit">µ</span><span class="dsi-unit">m</span></span></span></span>';
    expect(unitFourth.toHTML()).toBe(expectedHTML);
  });
});
